@@ -16,17 +16,14 @@
 (* Loading and installation of user-defined printer functions *)
 
 open Misc
-open Longident
-open Types
 
 (* Error report *)
 
-type error =
-  | Load_failure of Dynlink.error
-  | Unbound_identifier of Longident.t
-  | Unavailable_module of string * Longident.t
-  | Wrong_type of Longident.t
-  | No_active_printer of Longident.t
+type error = [
+    Topprinters.error
+  | `Load_failure of Dynlink.error
+  | `Unavailable_module of string * Longident.t
+]
 
 exception Error of error
 
@@ -41,8 +38,8 @@ let rec loadfiles ppf name =
     Dynlink.loadfile filename;
     let d = Filename.dirname name in
     if d <> Filename.current_dir_name then begin
-      if not (List.mem d (Load_path.get_paths ())) then
-        Load_path.add_dir d;
+      if not (List.mem d (Load_path.get_path_list ())) then
+        Load_path.add_dir ~hidden:false d;
     end;
     fprintf ppf "File %s loaded@."
       (if d <> Filename.current_dir_name then
@@ -52,7 +49,7 @@ let rec loadfiles ppf name =
     true
   with
   | Dynlink.Error (Dynlink.Unavailable_unit unit) ->
-      loadfiles ppf (String.uncapitalize_ascii unit ^ ".cmo")
+      loadfiles ppf (Unit_info.normalize unit ^ ".cmo")
         &&
       loadfiles ppf name
   | Not_found ->
@@ -62,7 +59,7 @@ let rec loadfiles ppf name =
       fprintf ppf "%s: %s@." name msg;
       false
   | Dynlink.Error e ->
-      raise(Error(Load_failure e))
+      raise(Error(`Load_failure e))
 
 let loadfile ppf name =
   ignore(loadfiles ppf name)
@@ -77,7 +74,9 @@ let rec eval_address = function
     let bytecode_or_asm_symbol = Ident.name id in
     begin match Dynlink.unsafe_get_global_value ~bytecode_or_asm_symbol with
     | None ->
-      raise (Symtable.Error (Symtable.Undefined_global bytecode_or_asm_symbol))
+      raise (Symtable.Error (Symtable.Undefined_global
+        (Symtable.Global.Glob_compunit (Cmo_format.Compunit
+          bytecode_or_asm_symbol))))
     | Some obj -> obj
     end
   | Env.Adot(addr, pos) -> Obj.field (eval_address addr) pos
@@ -90,81 +89,68 @@ let eval_value_path env path =
 
 (* Install, remove a printer (as in toplevel/topdirs) *)
 
-(* since 4.00, "topdirs.cmi" is not in the same directory as the standard
-  library, so we load it beforehand as it cannot be found in the search path. *)
-let init () =
-  let topdirs =
-    Filename.concat !Parameters.topdirs_path "topdirs.cmi" in
-  ignore (Env.read_signature "Topdirs" topdirs)
-
-let match_printer_type desc typename =
-  let printer_type =
-    try
-      Env.lookup_type (Ldot(Lident "Topdirs", typename)) Env.empty
-    with Not_found ->
-      raise (Error(Unbound_identifier(Ldot(Lident "Topdirs", typename)))) in
-  Ctype.begin_def();
-  let ty_arg = Ctype.newvar() in
-  Ctype.unify Env.empty
-    (Ctype.newconstr printer_type [ty_arg])
-    (Ctype.instance desc.val_type);
-  Ctype.end_def();
-  Ctype.generalize ty_arg;
-  ty_arg
-
-let find_printer_type lid =
-  try
-    let (path, desc) = Env.lookup_value lid Env.empty in
-    let (ty_arg, is_old_style) =
-      try
-        (match_printer_type desc "printer_type_new", false)
-      with Ctype.Unify _ ->
-        (match_printer_type desc "printer_type_old", true) in
-    (ty_arg, path, is_old_style)
-  with
-  | Not_found -> raise(Error(Unbound_identifier lid))
-  | Ctype.Unify _ -> raise(Error(Wrong_type lid))
-
-let install_printer ppf lid =
-  let (ty_arg, path, is_old_style) = find_printer_type lid in
-  let v =
-    try
-      eval_value_path Env.empty path
-    with Symtable.Error(Symtable.Undefined_global s) ->
-      raise(Error(Unavailable_module(s, lid))) in
-  let print_function =
-    if is_old_style then
-      (fun _formatter repr -> Obj.obj v (Obj.obj repr))
-    else
-      (fun formatter repr -> Obj.obj v formatter (Obj.obj repr)) in
-  Printval.install_printer path ty_arg ppf print_function
+(* Very close to Topdirs.install_printer_by_kind
+   except that we do fetch the (remote) values
+   {b and fallback if it fails} *)
+ let install_printer lid =
+  match Topprinters.find_printer Env.empty lid with
+  | Error error -> raise (Error (error :> error))
+  | Ok (path, kind) ->
+      let v =
+        try
+          eval_value_path Env.empty path
+        with Symtable.Error(Symtable.Undefined_global global) ->
+          let s = Symtable.Global.name global in
+          raise (Error (`Unavailable_module(s, lid))) in
+      let print_with_fallback ppf f remote_val =
+        try
+          f (Debugcom.Remote_value.obj remote_val)
+        with
+          Debugcom.Marshalling_error ->
+            fprintf ppf "<cannot fetch remote object>" in
+      match kind with
+      | Topprinters.Old ty_arg ->
+          let print_function ppf remote_val =
+            print_with_fallback ppf (Obj.obj v) remote_val in
+          Printval.install_printer path ty_arg print_function
+      | Topprinters.Simple ty_arg ->
+          let print_function ppf remote_val =
+            print_with_fallback ppf (Obj.obj v ppf) remote_val in
+          Printval.install_printer path ty_arg print_function
+      | Topprinters.Generic _ ->
+          raise (Error (`Wrong_type lid))
 
 let remove_printer lid =
-  let (_ty_arg, path, _is_old_style) = find_printer_type lid in
-  try
-    Printval.remove_printer path
-  with Not_found ->
-    raise(Error(No_active_printer lid))
+  match Topprinters.find_printer Env.empty lid with
+  | Error error -> raise (Error (error :> error))
+  | Ok (path, _kind) ->
+      try
+        Printval.remove_printer path
+      with Not_found ->
+        raise(Error(`No_active_printer path))
 
 (* Error report *)
 
 open Format
+module Style = Misc.Style
+let quoted_longident =
+  Format_doc.compat @@ Style.as_inline_code Printtyp.Doc.longident
 
 let report_error ppf = function
-  | Load_failure e ->
+  | `Load_failure e ->
       fprintf ppf "@[Error during code loading: %s@]@."
         (Dynlink.error_message e)
-  | Unbound_identifier lid ->
+  | `Unbound_identifier lid ->
       fprintf ppf "@[Unbound identifier %a@]@."
-      Printtyp.longident lid
-  | Unavailable_module(md, lid) ->
+        quoted_longident lid
+  | `Unavailable_module(md, lid) ->
       fprintf ppf
         "@[The debugger does not contain the code for@ %a.@ \
-           Please load an implementation of %s first.@]@."
-        Printtyp.longident lid md
-  | Wrong_type lid ->
+         Please load an implementation of %s first.@]@."
+        quoted_longident lid md
+  | `Wrong_type lid ->
       fprintf ppf "@[%a has the wrong type for a printing function.@]@."
-      Printtyp.longident lid
-  | No_active_printer lid ->
+        quoted_longident lid
+  | `No_active_printer path ->
       fprintf ppf "@[%a is not currently active as a printing function.@]@."
-      Printtyp.longident lid
+        Printtyp.path path
