@@ -16,10 +16,19 @@
 (** First-in first-out queues.
 
    This module implements queues (FIFOs), with in-place modification.
+   See {{!examples} the example section} below.
+*)
 
-   {b Warning} This module is not thread-safe: each {!Queue.t} value
-   must be protected from concurrent access (e.g. with a [Mutex.t]).
-   Failure to do so can lead to a crash.
+(** {b Unsynchronized accesses} *)
+
+[@@@alert unsynchronized_access
+    "Unsynchronized accesses to queues are a programming error."
+]
+
+(**
+    Unsynchronized accesses to a queue may lead to an invalid queue state.
+    Thus, concurrent accesses to queues must be synchronized (for instance
+    with a {!Mutex.t}).
 *)
 
 type !'a t
@@ -63,6 +72,11 @@ val peek_opt : 'a t -> 'a option
 val top : 'a t -> 'a
 (** [top] is a synonym for [peek]. *)
 
+val drop : 'a t -> unit
+(** [drop q] removes the first element in queue [q], or raises {!Empty}
+   if the queue is empty.
+   @since 5.3 *)
+
 val clear : 'a t -> unit
 (** Discard all elements from a queue. *)
 
@@ -80,7 +94,7 @@ val iter : ('a -> unit) -> 'a t -> unit
    from the least recently entered to the most recently entered.
    The queue itself is unchanged. *)
 
-val fold : ('b -> 'a -> 'b) -> 'b -> 'a t -> 'b
+val fold : ('acc -> 'a -> 'acc) -> 'acc -> 'a t -> 'acc
 (** [fold f accu q] is equivalent to [List.fold_left f accu l],
    where [l] is the list of [q]'s elements. The queue remains
    unchanged. *)
@@ -95,7 +109,7 @@ val transfer : 'a t -> 'a t -> unit
 
 val to_seq : 'a t -> 'a Seq.t
 (** Iterate on the queue, in front-to-back order.
-    The behavior is not defined if the queue is modified
+    The behavior is not specified if the queue is modified
     during the iteration.
     @since 4.07 *)
 
@@ -106,3 +120,98 @@ val add_seq : 'a t -> 'a Seq.t -> unit
 val of_seq : 'a Seq.t -> 'a t
 (** Create a queue from a sequence.
     @since 4.07 *)
+
+(** {1:examples Examples}
+
+  {2 Basic Example}
+
+   A basic example:
+    {[
+    # let q = Queue.create ()
+    val q : '_weak1 Queue.t = <abstr>
+
+
+    # Queue.push 1 q; Queue.push 2 q; Queue.push 3 q
+    - : unit = ()
+
+    # Queue.length q
+    - : int = 3
+
+    # Queue.pop q
+    - : int = 1
+
+    # Queue.pop q
+    - : int = 2
+
+    # Queue.pop q
+    - : int = 3
+
+    # Queue.pop q
+    Exception: Stdlib.Queue.Empty.
+    ]}
+
+  {2 Search Through a Graph}
+
+   For a more elaborate example, a classic algorithmic use of queues
+   is to implement a BFS (breadth-first search) through a graph.
+
+   {[
+     type graph = {
+       edges: (int, int list) Hashtbl.t
+     }
+
+    (* Search in graph [g] using BFS, starting from node [start].
+       It returns the first node that satisfies [p], or [None] if
+       no node reachable from [start] satisfies [p].
+    *)
+    let search_for ~(g:graph) ~(start:int) (p:int -> bool) : int option =
+      let to_explore = Queue.create() in
+      let explored = Hashtbl.create 16 in
+
+      Queue.push start to_explore;
+      let rec loop () =
+        if Queue.is_empty to_explore then None
+        else
+          (* node to explore *)
+          let node = Queue.pop to_explore in
+          explore_node node
+
+      and explore_node node =
+        if not (Hashtbl.mem explored node) then (
+          if p node then Some node (* found *)
+          else (
+            Hashtbl.add explored node ();
+            let children =
+              Hashtbl.find_opt g.edges node
+              |> Option.value ~default:[]
+            in
+            List.iter (fun child -> Queue.push child to_explore) children;
+            loop()
+          )
+        ) else loop()
+      in
+      loop()
+
+    (* a sample graph *)
+    let my_graph: graph =
+      let edges =
+        List.to_seq [
+          1, [2;3];
+          2, [10; 11];
+          3, [4;5];
+          5, [100];
+          11, [0; 20];
+        ]
+        |> Hashtbl.of_seq
+      in {edges}
+
+    # search_for ~g:my_graph ~start:1 (fun x -> x = 30)
+    - : int option = None
+
+    # search_for ~g:my_graph ~start:1 (fun x -> x >= 15)
+    - : int option = Some 20
+
+    # search_for ~g:my_graph ~start:1 (fun x -> x >= 50)
+    - : int option = Some 100
+   ]}
+   *)
