@@ -1,5 +1,5 @@
 (* TEST
-modules = "f.ml"
+ modules = "f.ml";
 *)
 
 exit (if F.Alias = F.alias then 0 else 1)
